/*
             LUFA Library
     Copyright (C) Dean Camera, 2012.

  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/

/*
  Copyright 2012  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *
 *  Main source file for the MediaControl project. This file contains the main tasks of
 *  the demo and is responsible for the initial application hardware configuration.
 */

#include "MediaController.h"

/* PC7 */
#define LED_ON  { PORTC |=   (1 << 7); }
#define LED_OFF { PORTC &= ~ (1 << 7); }

/* PD5 = TXLED */
#define LED2_ON  {PORTD = PORTD & (~ (1 << 5)); }
#define LED2_OFF {PORTD = (PORTD | (1 << 5)); }

/* PB0 = RXLED */
#define LED3_ON  {PORTB = PORTB & (~ (1 << 0)); }
#define LED3_OFF {PORTB = PORTB | (1 << 0); }

/* D5 (PC6) */
#define LED_PL_ON  { PORTC = PORTC | (1 << 6); }
#define LED_PL_OFF { PORTC = PORTC & ( ~ (1 << 6)); }

/* D6 (PD7) */
#define LED_MUTE_ON   { PORTD = PORTD | (1 << 7); }
#define LED_MUTE_OFF  { PORTD = PORTD & ( ~ (1 << 7)); }
#define LED_MUTE_FLIP { PORTD = PORTD ^ (1 << 7); }

/* blink interval */
#define BLINK_COUNT_MAX 170

/** Buffer to hold the previously generated Media Control HID report, for comparison purposes inside the HID class driver. */
static uint8_t PrevMediaControlHIDReportBuffer[sizeof(USB_MediaReport_Data_t)];

/** LUFA HID Class driver interface configuration and state information. This structure is
 *  passed to all HID Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_HID_Device_t MediaControl_HID_Interface =
 	{
		.Config =
			{
				.InterfaceNumber              = 0,
				.ReportINEndpoint             =
					{
						.Address              = MEDIACONTROL_HID_EPADDR,
						.Size                 = MEDIACONTROL_HID_EPSIZE,
						.Banks                = 1,
					},
				.PrevReportINBuffer           = PrevMediaControlHIDReportBuffer,
				.PrevReportINBufferSize       = sizeof(PrevMediaControlHIDReportBuffer),
			},
    };


/** Main program entry point. This routine contains the overall program flow, including initial
 *  setup of all components and the main program loop.
 */
int main(void)
{
	SetupHardware();
	sei();

	for (;;)
	{
		HID_Device_USBTask(&MediaControl_HID_Interface);
		USB_USBTask();
	}
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware()
{
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
	clock_prescale_set(clock_div_1);

	/* Hardware Initialization */
	/* LED PORT for test */
	DDRC |= 0b10000000;	/* PC7 = D13 = LED */
	LED_ON;
	LED_OFF;
	/* extra led TXLED */
	DDRD = DDRD | (1 << 5);	/* PD5 */
	LED2_ON;
	LED2_OFF;
	
	DDRB = DDRB | (1 << 0);	/* PB0 */
	LED3_ON;
	LED3_OFF;

	DDRC = DDRC | (1 << 6);	/* D5(PC6) */
	LED_PL_ON;

	DDRD = DDRD | (1 << 7);	/* D6(PD7) */
	LED_MUTE_OFF;
	
	USB_Init();

	/* KEY */
	DDRD = DDRD & (~ 0b00010011);	/* INPUT : PD0 = D3, PD1 = D2, PD4 = D4 */
	PORTD =  PORTD | 0b00010011;	/* PULL UP */
}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
  // LEDs_SetAllLEDs(LEDMASK_USB_ENUMERATING);
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
  // LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= HID_Device_ConfigureEndpoints(&MediaControl_HID_Interface);

	USB_Device_EnableSOFEvents();

}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void)
{
	HID_Device_ProcessControlRequest(&MediaControl_HID_Interface);
}

/** Event handler for the USB device Start Of Frame event. */
void EVENT_USB_Device_StartOfFrame(void)
{
	HID_Device_MillisecondElapsed(&MediaControl_HID_Interface);
}

/** HID class driver callback function for the creation of HID reports to the host.
 *
 *  \param[in]     HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in,out] ReportID    Report ID requested by the host if non-zero, otherwise callback should set to the generated report ID
 *  \param[in]     ReportType  Type of the report to create, either HID_REPORT_ITEM_In or HID_REPORT_ITEM_Feature
 *  \param[out]    ReportData  Pointer to a buffer where the created report should be stored
 *  \param[out]    ReportSize  Number of bytes written in the report (or zero if no report is to be sent)
 *
 *  \return Boolean true to force the sending of the report, false to let the library determine if it needs to be sent
 */
bool CALLBACK_HID_Device_CreateHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo, uint8_t* const ReportID,
                                         const uint8_t ReportType, void* ReportData, uint16_t* const ReportSize)
{
	USB_MediaReport_Data_t* MediaReport = (USB_MediaReport_Data_t*)ReportData;

	uint8_t UsedKeyCodes = 0;
	static uint8_t dataRead = 0;
	static uint8_t state_mute = 0;
	static int8_t mute_cnt = 10;
	static uint8_t blink_cnt = 0;
	int8_t direction;
	//	static const int dir[] = { 0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0 }; /* 回転方向テーブル */
	static const int dir[] = { 0,1,-1,0,0,0,0,0,0,0,0,0,0,0,0,0 }; /* 回転方向テーブル(１クリック１アップダウン） */

	dataRead = (dataRead << 2) + (PIND & 0b00000011);  /* 前回値と今回値でインデックスとする */
	direction = dir[dataRead & 15];

	MediaReport->Mute = false;
	MediaReport->VolumeUp = false;
	MediaReport->VolumeDown  = false;

	if (direction == 1) {
	  MediaReport->VolumeUp = true;
	  LED2_ON;
	  LED3_OFF;
	  state_mute = 0;
	  LED_PL_ON;
	  LED_MUTE_OFF;
	} else if (direction == -1) {
	  MediaReport->VolumeDown  = true;
	  LED2_OFF;
	  LED3_ON;
	  state_mute = 0;
	  LED_PL_ON;
	  LED_MUTE_OFF;
	}

	if ((PIND & 0x10) == 0 ) { /* PD4 for Mute */
	  if (mute_cnt > 0) {
	    mute_cnt--;
	    if (mute_cnt == 0) {
	      MediaReport->Mute = true;
	      mute_cnt = -1;
	      if (state_mute == 0) {
		state_mute = 1;
		LED2_OFF;
		LED3_OFF;
		LED_PL_OFF;
		LED_MUTE_ON;
	      } else {
		state_mute = 0;
		LED2_ON;
		LED3_ON;
		LED_PL_ON;
		LED_MUTE_OFF;
	      }
	    }
	  }
	} else {
	  mute_cnt = 10;
	}

	if (state_mute == 1) {
	  if ((--blink_cnt) == 0) {
	    LED_MUTE_FLIP;
	    blink_cnt = BLINK_COUNT_MAX;
	  }
	} else {
	  blink_cnt = BLINK_COUNT_MAX;
	}

	*ReportSize = sizeof(USB_MediaReport_Data_t);
	return false;
}

/** HID class driver callback function for the processing of HID reports from the host.
 *
 *  \param[in] HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in] ReportID    Report ID of the received report from the host
 *  \param[in] ReportType  The type of report that the host has sent, either HID_REPORT_ITEM_Out or HID_REPORT_ITEM_Feature
 *  \param[in] ReportData  Pointer to a buffer where the received report has been stored
 *  \param[in] ReportSize  Size in bytes of the received HID report
 */
void CALLBACK_HID_Device_ProcessHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                          const uint8_t ReportID,
                                          const uint8_t ReportType,
                                          const void* ReportData,
                                          const uint16_t ReportSize)
{
	// Unused (but mandatory for the HID class driver) in this demo, since there are no Host->Device reports
}

